#!/bin/bash
# Script to generate HTML Sitemap based on urllist.txt
# Copyright Ben Tasker 2009
# Released under the GNU GPL

# Slightly adapted version, creates a seperate section for BASH and Perl Scripts 
# Basically anything ending with .sh or .pl

# HTML_gen V1.5
# Released 23 Jan 2010 under GNU GPL V3
# See http://benscomputer.no-ip.org/LICENSE3
#

# Added Lockfile ability
# Creates list of 'Premium' pages. Identifies pages by looking for the
# string prem_ in the URI


# Variables


# Path to Url listing
URLLIST="/mnt/.apacheroot/www/localhost/htdocs/urllist.txt"
# Path to Sitemap Template
TEMPLATE="/mnt/.apacheroot/www/localhost/htdocs/sitemap_template.html"
# Where should the new sitemap be placed
SITEMAPLOCAT="/mnt/.apacheroot/www/localhost/htdocs/sitemap.shtml"

# Where should the Premium Articles Index be stored

PREMIUMSITEMAP="/mnt/.apacheroot/www/localhost/htdocs/Premium_Articles.shtml"

# Where's the Premium Template
PREMIUMTEMPLATE="/mnt/.apacheroot/www/localhost/htdocs/prem_sitemap_template.html"

# Do you want the user to be able to see when the page was last generated?
# If not set this to N. It'll still be added to the HTML but only be visible
# with View Source
GENTIMEVISIBLE="N"




# Code starts here!


# Move to our working directory
cd /tmp


# Check for the existence of a Lockfile
if [ -e HTML_sitemap_gen.lockfile ]
then
# Process has been locked - Exit

echo "Found lockfile - Exiting"
exit

fi

# Create the lockfile to prevent further instances being run

echo "" > HTML_sitemap_gen.lockfile
chmod 777 HTML_sitemap_gen.lockfile


# Are we printing Generation Times
if [ "$GENTIMEVISIBLE" == "N" ]
then

GENTIMEVIEWOPEN="<!--"
GENTIMEVIEWCLOSE="-->"
else
GENTIMEVIEWOPEN="<br><br>"
fi



# Prepare the Template Header
cp $TEMPLATE ./sitemaptemplate.html
# Strip Newlines to make processing easier
tr -d "\n" < sitemaptemplate.html > sitemaptmp
# Now lets just take the header
PAGEHEAD=$(cat sitemaptmp | awk -F "<!-- HEADER -->" '{print $2;}')
echo $PAGEHEAD > sitemap.html
GENTIME=$( date +'%D %H:%M:%S' )
echo "$GENTIMEVIEWOPEN Sitemap last Generated $GENTIME $GENTIMEVIEWCLOSE" >> sitemap.html
echo "<br><br><b>Content Types</b><br><br>" >> sitemap.html
echo "<li><a href=#HTML>HTML</a></li>" >> sitemap.html
echo "<li><a href=#PDF>PDF</a></li>" >> sitemap.html
echo "<li><a href=#TXT>Plain Text</a></li>" >> sitemap.html
echo "<li><a href=#IMAGES>Images</a></li>" >> sitemap.html
echo "<li><a href=#Scripts>Scripts and Programs</a></li>" >> sitemap.html
echo "<li><a href=#ODOCS>Other Document Types</a></li>" >> sitemap.html
echo "<br><br>" >> sitemap.html
echo "" >> sitemap.html
echo "<!-- Please do not remove these comments -->" >> sitemap.html
echo "<!-- Generated using HTML_Sitemap_Gen_V0.1 -->" >> sitemap.html
echo "<!-- Released under the GNU GPL -->" >> sitemap.html
echo "<!-- To get your copy, visit" >> sitemap.html
echo "http://benscomputer.no-ip.org/projects.html#Sitemapgen -->" >> sitemap.html
echo "" >> sitemap.html
# URL Processing
# Lets put the HTML pages first
echo "<a name=HTML></a><br><br><b>HTML</b><br><br>" >> sitemap.html
for i in $( cat $URLLIST )
do
BYPASS="N"
FILEFAIL="N"

# Is it a Premium Article? If so we want to add it to the Prem list
# Premium article filenames start with prem_

echo "$i" | grep "prem_" > /dev/null
if [ "$?" == "0" ]
then

echo "$i"  >> .prem_articles.list
fi


# Sometimes wget igonores the A option because a link is specified, so lets do a quick check on the address
echo "$i" | grep '[.]html' > /dev/null
A=$?
echo "$i" | grep '[.]shtml' > /dev/null
B=$?
echo "$i" | grep '[.]php' > /dev/null
C=$?
echo "$i" | grep '[.]asp' > /dev/null 
D=$?
echo "$i" | grep '[.]htm' > /dev/null
E=$?
echo "$i" | grep '[.]xhtml' > /dev/null
F=$?
# This could be done in less lines with a long RegExp but I've got a bit of a headache!

# Ok so we know that we have run 6 checks, if the file contains one of those filename extensions the sum of all of them should be
# 5, grep returns 0 when it finds a string and 1 when it doesn't. So if the total is 6 it didn't find any
CHECKING=$(( $A + $B + $C + $D + $E + $F))
if [ "$CHECKING" == "6" ]
then
# File does not have a recognised extension
echo "Non HTML/SHTML/XHTML/ASP/PHP file found at $i"
echo "Skipping"
FILEFAIL=Y
fi

if [ "$FILEFAIL" == "N" ]
then
# Get the page
wget -U "HTML Sitemap Gen" -q -O tmpfile "$i";

# Check the page actually existed, wget returns 0 for success and 1 for failure
if [ "$?" == "1" ]
then
# It failed
BYPASS="Y"
fi
if [ "$BYPASS" == "N" ]
then
# Find the Title line (it may span more than one)
cat tmpfile | grep "title>" > tmpfile2
# Strip out any newlines
tr -d "\n" < tmpfile2 > tmpfile
# now get the text between <title> and </title>
TitleText=$(cat tmpfile | awk -F "title>" '{print $2;}')
echo "$TitleText" > tmpfile2
# Tmpfile now contains the title byt it has the suffix <z
#Lets strip that out now
sed s/"<\/"//g tmpfile2 > tmpfile
# OK now lets turn that file into a variable
PAGETITLE=$( cat tmpfile )
# Now lets generate the sitemap to add it into the sitemap
echo -n '<br><a href="' >> sitemap.html
# Insert the link
echo -n "$i" >> sitemap.html
echo -n '" target=_blank>' >> sitemap.html
# Insert the title
echo -n "$PAGETITLE </a>" >> sitemap.html
# OK it's inserted
# But to keep the file readable put a newline into the file
echo "" >> sitemap.html
# Now lets tidy up before generating the next bit
rm -f tmpfile
rm -f tmpfile2

else

# We are bypassing because the Wget operation failed
echo "Retrieval of $i failed"
echo "$i has not been included in the sitemap"

fi
else
# File type was wrong
echo "File $i has not been inserted into sitemap"
fi
done

# URL Processing
# Lets put the PDF's Next
echo "<a name=PDF></a><br><br><br><b>PDF</b><br><br>" >> sitemap.html
for i in $( cat $URLLIST )
do
BYPASS="N"
FILEFAIL="N"

# Sometimes wget igonores the A option because a link is specified, so lets do a quick check on the address
echo "$i" | grep '[.]pdf' > /dev/null
A=$?
echo "$i" | grep '[.]PDF' > /dev/null
B=$?
# This could be done in less lines with a long RegExp but I've got a bit of a headache!

CHECKING=$(( $A + $B ))
if [ "$CHECKING" == "2" ]
then
# File does not have a recognised extension
echo "Non PDF file found at $i"
echo "Skipping"
FILEFAIL=Y
fi

if [ "$FILEFAIL" == "N" ]
then
# Get the page
wget -U "HTML Sitemap Gen" -q -O tmpfile "$i";

# Check the page actually existed, wget returns 0 for success and 1 for failure
if [ "$?" == "1" ]
then
# It failed
BYPASS="Y"
fi
if [ "$BYPASS" == "N" ]
then
# Can't find the Title so easily in a PDf so lets just post the link
echo -n '<br><a href="' >> sitemap.html
# Insert the link
echo -n "$i" >> sitemap.html
echo -n '" target=_blank>' >> sitemap.html
# Insert the title
echo -n "$i </a>" >> sitemap.html
# OK it's inserted
# But to keep the file readable put a newline into the file
echo "" >> sitemap.html
# Now lets tidy up before generating the next bit
rm -f tmpfile
rm -f tmpfile2

else

# We are bypassing because the Wget operation failed
echo "Retrieval of $i failed"
echo "$i has not been included in the sitemap"

fi
else
# File type was wrong
echo "File $i has not been inserted into sitemap on this pass"
fi
done

# URL Processing
# Lets put the Text Files Next
echo "<a name=TXT></a><br><br><b>Plain Text</b><br><br>" >> sitemap.html
for i in $( cat $URLLIST )
do
BYPASS="N"
FILEFAIL="N"

# Sometimes wget igonores the A option because a link is specified, so lets do a quick check on the address
echo "$i" | grep '[.]txt' > /dev/null
A=$?
echo "$i" | grep '[.]TXT' > /dev/null
B=$?
# This could be done in less lines with a long RegExp but I've got a bit of a headache!

CHECKING=$(( $A + $B ))
if [ "$CHECKING" == "2" ]
then
# File does not have a recognised extension
echo "Non Text file found at $i"
echo "Skipping"
FILEFAIL=Y
fi

if [ "$FILEFAIL" == "N" ]
then
# Get the page
wget -U "HTML Sitemap Gen" -q -O tmpfile "$i";

# Check the page actually existed, wget returns 0 for success and 1 for failure
if [ "$?" == "1" ]
then
# It failed
BYPASS="Y"
fi
if [ "$BYPASS" == "N" ]
then
# Can't find the Title so easily in a PDf so lets just post the link
echo -n '<br><a href="' >> sitemap.html
# Insert the link
echo -n "$i" >> sitemap.html
echo -n '" target=_blank>' >> sitemap.html
# Insert the title
echo -n "$i </a>" >> sitemap.html
# OK it's inserted
# But to keep the file readable put a newline into the file
echo "" >> sitemap.html
# Now lets tidy up before generating the next bit
rm -f tmpfile
rm -f tmpfile2

else

# We are bypassing because the Wget operation failed
echo "Retrieval of $i failed"
echo "$i has not been included in the sitemap"

fi
else
# File type was wrong
echo "File $i has not been inserted into sitemap on this pass"
fi
done

# URL Processing
# Lets put the Image Files Next
echo "<a name=IMAGES></a><br><br><b>Images</b><br><br>" >> sitemap.html
for i in $( cat $URLLIST )
do
BYPASS="N"
FILEFAIL="N"

# Sometimes wget igonores the A option because a link is specified, so lets do a quick check on the address
echo "$i" | grep -i '[.]gif' > /dev/null
A=$?
echo "$i" | grep -i '[.]jpg' > /dev/null
B=$?
echo "$i" | grep -i '[.]jpeg' > /dev/null
C=$?
echo "$i" | grep -i '[.]png' > /dev/null
D=$?
echo "$i" | grep -i '[.]bmp' > /dev/null
E=$?
echo "$i" | grep -i '[.]svc' > /dev/null
F=$?
echo "$i" | grep -i '[.]psd' > /dev/null
G=$?


# This could be done in less lines with a long RegExp but I've got a bit of a headache!

CHECKING=$(( $A + $B + $C +$D +$E +$F + $G ))
if [ "$CHECKING" == "7" ]
then
# File does not have a recognised extension
echo "Non Image file found at $i"
echo "Skipping"
FILEFAIL=Y
fi

if [ "$FILEFAIL" == "N" ]
then
# Get the page
wget -U "HTML Sitemap Gen" -q -O tmpfile "$i";

# Check the page actually existed, wget returns 0 for success and 1 for failure
if [ "$?" == "1" ]
then
# It failed
BYPASS="Y"
fi
if [ "$BYPASS" == "N" ]
then
# Can't find the Title so easily in a PDf so lets just post the link
echo -n '<br><a href="' >> sitemap.html
# Insert the link
echo -n "$i" >> sitemap.html
echo -n '" target=_blank>' >> sitemap.html
# Insert the title
echo -n "$i </a>" >> sitemap.html
# OK it's inserted
# But to keep the file readable put a newline into the file
echo "" >> sitemap.html
# Now lets tidy up before generating the next bit
rm -f tmpfile
rm -f tmpfile2

else

# We are bypassing because the Wget operation failed
echo "Retrieval of $i failed"
echo "$i has not been included in the sitemap"

fi
else
# File type was wrong
echo "File $i has not been inserted into sitemap  on this pass"
fi
done

echo "<a name=Scripts></a><br><br><b>Scripts and Programs</b><br><Br>" >> sitemap.html
for i in $( cat $URLLIST )
do
BYPASS="N"
FILEFAIL="N"

# Sometimes wget igonores the A option because a link is specified, so lets do a quick check on the address
echo "$i" | grep -i '[.]sh' > /dev/null
A=$?
echo "$i" | grep -i '[.]SH' > /dev/null
B=$?
echo "$i" | grep -i '[.]pl' > /dev/null
C=$?
echo "$i" | grep -i '[.]PL' > /dev/null
D=$?
echo "$i" | grep -i '[.]tar.gz' > /dev/null
E=$?

# This could be done in less lines with a long RegExp but I've got a bit of a headache!

CHECKING=$(( $A + $B + $C + $D + $E ))

echo "$i" | grep -i '[.]shtml' > /dev/null
if [ "$?" == "0" ]
then
CHECKING="5"
fi

if [ "$CHECKING" == "5" ]
then
# File does not have a recognised extension
echo "Non Script file found at $i"
echo "Skipping"
FILEFAIL=Y
fi

if [ "$FILEFAIL" == "N" ]
then
# Get the page
wget -U "HTML Sitemap Gen" -q -O tmpfile "$i";

# Check the page actually existed, wget returns 0 for success and 1 for failure
if [ "$?" == "1" ]
then
# It failed
BYPASS="Y"
fi
if [ "$BYPASS" == "N" ]
then
# Can't find the Title so easily in a PDf so lets just post the link
echo -n '<br><a href="' >> sitemap.html
# Insert the link
echo -n "$i" >> sitemap.html
echo -n '" target=_blank>' >> sitemap.html
# Insert the title
echo -n "$i </a>" >> sitemap.html
# OK it's inserted
# But to keep the file readable put a newline into the file
echo "" >> sitemap.html
# Now lets tidy up before generating the next bit
rm -f tmpfile
rm -f tmpfile2

else

# We are bypassing because the Wget operation failed
echo "Retrieval of $i failed"
echo "$i has not been included in the sitemap"

fi
else
# File type was wrong
echo "File $i has not been inserted into sitemap  on this pass"
fi
done







# URL Processing
# Rarely post Documents from OpenOffice/Office, but just in case
echo "<a name=ODOCS></a><br><br><b>Other Documents</b><br><br>" >> sitemap.html
for i in $( cat $URLLIST )
do
BYPASS="N"
FILEFAIL="N"

# We want all other file types to be listed here, so lets exclude the ones already indexed
echo "$i" | grep -iv '[.]html' > /dev/null
A=$?
echo "$i" | grep -iv '[.]htm' > /dev/null
B=$?
echo "$i" | grep -iv '[.]asp' > /dev/null
C=$?
echo "$i" | grep -iv '[.]php' > /dev/null
D=$?
echo "$i" | grep -iv '[.]pdf' > /dev/null
E=$?
echo "$i" | grep -iv '[.]jpg' > /dev/null
F=$?
echo "$i" | grep -iv '[.]gif' > /dev/null
G=$?
echo "$i" | grep -iv '[.]jpeg' > /dev/null
H=$?
echo "$i" | grep -iv '[.]png' > /dev/null
I=$?
echo "$i" | grep -iv '[.]bmp' > /dev/null
J=$?
echo "$i" | grep -iv '[.]txt' > /dev/null
K=$?
echo "$i" | grep -iv '[.]shtml' > /dev/null
L=$?
echo "$i" | grep -iv '[.]xhtml' > /dev/null
M=$?
echo "$i" | grep -iv '[.]svc' > /dev/null
N=$?
echo "$i" | grep -iv '[.]psd' > /dev/null
O=$?
echo "$i" | grep -iv '[.]sh' > /dev/null
P=$?
echo "$i" | grep -iv '[.]SH' > /dev/null
Q=$?
echo "$i" | grep -iv '[.]pl' > /dev/null
R=$?
echo "$i" | grep -iv '[.]PL' > /dev/null
S=$?
echo "$i" | grep -iv ".tar.gz" > /dev/null
T=$?

# This could be done in less lines with a long RegExp but I've got a bit of a headache!
# Wget returns 1 if it doesn't find the string,so if CHECKING is less that 15 it found one

CHECKING=$(( $A + $B + $C +$D + $E + $F + $G + $H + $I + $J + $K + $L + $M + $N + $O + $P + $Q + $R + $S + $T ))
if [ "$CHECKING" == "0" ]
then


if [ "$FILEFAIL" == "N" ]
then
# Get the page
wget -U "HTML Sitemap Gen" -q -O tmpfile "$i";

# Check the page actually existed, wget returns 0 for success and 1 for failure
if [ "$?" == "1" ]
then
# It failed
BYPASS="Y"
fi
if [ "$BYPASS" == "N" ]
then
# Can't find the Title so easily in a PDf so lets just post the link
echo -n '<br><a href="' >> sitemap.html
# Insert the link
echo -n "$i" >> sitemap.html
echo -n '" target=_blank>' >> sitemap.html
# Insert the title
echo -n "$i </a>" >> sitemap.html
# OK it's inserted
# But to keep the file readable put a newline into the file
echo "" >> sitemap.html
# Now lets tidy up before generating the next bit
rm -f tmpfile
rm -f tmpfile2

else

# We are bypassing because the Wget operation failed
echo "Retrieval of $i failed"
echo "$i has not been included in the sitemap"

fi
else
# File type was wrong
echo "File $i has not been inserted into sitemap on this pass"
fi
else
# File does not have a recognised extension
echo "Previously indexed Filetype found at $i"
echo "Skipping"
FILEFAIL=Y
fi
done

echo "" >> sitemap.html
echo "<!-- Please do not remove these comments -->" >> sitemap.html
echo "<!-- Generated using HTML_Sitemap_Gen_V0.1 -->" >> sitemap.html
echo "<!-- Released under the GNU GPL -->" >> sitemap.html
echo "<!-- To get your copy, visit" >> sitemap.html
echo "http://benscomputer.no-ip.org/projects.html#Sitemapgen -->" >> sitemap.html
echo "" >> sitemap.html

# Bung the footer on the end
PAGEFOOT=$(cat sitemaptmp | awk -F "<!-- FOOTER -->" '{print $2;}')
echo $PAGEFOOT >> sitemap.html

rm -f tmpfile 2> /dev/null



# Now create the list of Premium Articles




# Prepare the Template Header
cp $PREMIUMTEMPLATE ./premsitemaptemplate.html

# Strip Newlines to make processing easier
tr -d "\n" < premsitemaptemplate.html > premsitemaptmp



# Now lets just take the header
PAGEHEAD=$(cat premsitemaptmp | awk -F "<!-- HEADER -->" '{print $2;}')

# Store it
echo $PAGEHEAD > premsitemap.html

# When was it last generated
GENTIME=$( date +'%D %H:%M:%S' )
echo "$GENTIMEVIEWOPEN Index last Generated $GENTIME <br><br>$GENTIMEVIEWCLOSE" >> premsitemap.html
echo "Premium Articles are free to view, but you cannot copy, redistribute or adapt them without prior written permission" >> premsitemap.html
echo "<br><br>" >> premsitemap.html
echo "" >> premsitemap.html
echo "<!-- Please do not remove these comments -->" >> premsitemap.html
echo "<!-- Generated using HTML_Sitemap_Gen_V0.1 -->" >> premsitemap.html
echo "<!-- Released under the GNU GPL -->" >> premsitemap.html
echo "<!-- To get your copy, visit" >> premsitemap.html
echo "http://benscomputer.no-ip.org/projects.html#Sitemapgen -->" >> premsitemap.html
echo "" >> premsitemap.html



# URL Processing

for i in $( cat prem_articles.list )
do

####################

# Get the page
wget -U "HTML Sitemap Gen - Generating Premium Article List" -q -O tmpfile "$i";

# Find the Title line (it may span more than one)
cat tmpfile | grep "title>" > tmpfile2
# Strip out any newlines
tr -d "\n" < tmpfile2 > tmpfile
# now get the text between <title> and </title>
TitleText=$(cat tmpfile | awk -F "title>" '{print $2;}')
echo "$TitleText" > tmpfile2
# Tmpfile now contains the title byt it has the suffix <z
#Lets strip that out now
sed s/"<\/"//g tmpfile2 > tmpfile
# OK now lets turn that file into a variable
PAGETITLE=$( cat tmpfile )
# Now lets generate the sitemap to add it into the sitemap
echo -n '<br><a href="' >> premsitemap.html
# Insert the link
echo -n "$i" >> premsitemap.html
echo -n '">' >> premsitemap.html
# Insert the title
echo -n "$PAGETITLE</a>" >> premsitemap.html
# OK it's inserted
# But to keep the file readable put a newline into the file
echo "" >> premsitemap.html
# Now lets tidy up before generating the next bit
rm -f tmpfile
rm -f tmpfile2

done

# Add the footer of the page

echo "" >> premsitemap.html
echo "<!-- Please do not remove these comments -->" >> premsitemap.html
echo "<!-- Generated using HTML_Sitemap_Gen_V0.1 -->" >> premsitemap.html
echo "<!-- Released under the GNU GPL -->" >> premsitemap.html
echo "<!-- To get your copy, visit" >> premsitemap.html
echo "http://benscomputer.no-ip.org/projects.html#Sitemapgen -->" >> premsitemap.html
echo "" >> premsitemap.html

# Bung the footer on the end
PAGEFOOT=$(cat premsitemaptmp | awk -F "<!-- FOOTER -->" '{print $2;}')
echo $PAGEFOOT >> premsitemap.html

# Tidy up from our 'for' loop

rm -f tmpfile 2> /dev/null




#################
#    Tidy Up!   #
#################

rm -f sitemaptmp 2> /dev/null
rm -f premsitemaptmp 2> /dev/null
rm -f prem_articles.list 2> /dev/null


# Move the sitemap to the desired place
mv sitemap.html $SITEMAPLOCAT

# Now Move the Premium Sitemap to the desired Place
mv premsitemap.html $PREMIUMSITEMAP


# Remove the lock file

rm -f HTML_sitemap_gen.lockfile


# Report Back
echo "Finished!"


# Scram!!!
exit
